//---------------------------------------------------------------------------
#include "StochasticMort.h"
#include "MortalityOrg.h"
#include "ParsingFunctions.h"
#include "TreePopulation.h"
#include "SimManager.h"
#include "ModelMath.h"
#include <math.h>
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
// Constructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
clStochasticMort::clStochasticMort(clSimManager *p_oSimManager) :
      clWorkerBase( p_oSimManager ), clBehaviorBase( p_oSimManager ), clMortalityBase(p_oSimManager) 
{
 try {

   strcpy(m_cNameString, "stochasticmortshell");

   mp_fRandomMort = NULL;
 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clStochasticMort::clStochasticMort");
   throw(stcErr);
 }
}

////////////////////////////////////////////////////////////////////////////
// Destructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
clStochasticMort::~clStochasticMort() {
 delete[] mp_fRandomMort;
}

////////////////////////////////////////////////////////////////////////////
// SetNameData()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
void clStochasticMort::SetNameData(char *cNameString) {
 if (strcmp(cNameString, "adultstochasticmort") == 0)
   m_bIsAdult = true;
 else
   m_bIsAdult = false;
}

////////////////////////////////////////////////////////////////////////////
// DoShellSetup()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
void clStochasticMort::DoShellSetup(xercesc::DOMDocument *p_oDoc) {
 try {
   clPopulationBase *p_oTemp =
           mp_oSimManager->GetPopulationObject("treepopulation");
   clTreePopulation *p_oPop = (clTreePopulation*) p_oTemp;
   DOMElement *p_oElement = p_oDoc->getDocumentElement();
   floatVal *p_fTempValues;  //for getting species-specific values
   float fNumberYearsPerTimestep = mp_oSimManager->GetNumberOfYearsPerTimestep();
   short int iNumSpecies = mp_oMortalityOrg->GetNumberOfSpecies(),
             i; //loop counter

   //Declare the temp array and populate it with the species to which this
   //behavior applies
   p_fTempValues = new floatVal[m_iNumBehaviorSpecies];
   for (i = 0; i < m_iNumBehaviorSpecies; i++)
     p_fTempValues[i].code = mp_iWhatSpecies[i];

   //Declare the arrays for holding the variables
   mp_fRandomMort = new float[iNumSpecies];

   //Capture the values from the parameter file

   //Random mortality
   if (m_bIsAdult)
     FillSpeciesSpecificValue(p_oElement, "mo_nonSizeDepMort",
               "mo_nsdmVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true);
   else
     FillSpeciesSpecificValue(p_oElement, "mo_randomJuvenileMortality",
               "mo_rjmVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true);
   //Transfer to the appropriate array buckets and compound by the number
   //of years per timestep
   for (i = 0; i < m_iNumBehaviorSpecies; i++) {
     mp_fRandomMort[p_fTempValues[i].code] = 1 - pow(1 - p_fTempValues[i].val, fNumberYearsPerTimestep);
   }

   delete[] p_fTempValues;
 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clStochasticMort::DoShellSetup");
   throw(stcErr);
 }
}

///////////////////////////////////////////////////////////////////////////
// DoMort()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
///////////////////////////////////////////////////////////////////////////
bool clStochasticMort::DoMort(clTree *p_oTree, const float &fDiam,
     const short int &iSpecies) {
 try {

   if(clModelMath::GetRand() < mp_fRandomMort[iSpecies])
     return true;
   else
     return false;

 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clStochasticMort::DoMort");
   throw(stcErr);
 }
}
